﻿Imports System
Imports System.Drawing
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.ComponentModel.Design

Namespace System.Windows.Forms.Design

   Public Class CaptionPanelDesigner : Inherits ParentControlDesigner

      Protected Overrides Sub OnDragOver(ByVal e As DragEventArgs)
         If Me.Control.Controls.Count > 0 Then
            e.Effect = DragDropEffects.None
            Return
         End If
         MyBase.OnDragOver(e)
      End Sub

   End Class 'CaptionPanelDesigner

   Public Class SizerPanelDesigner : Inherits ParentControlDesigner

      Private Class LayoutOptions

         Private _TopDown As Boolean = Nothing
         <Description("True: FlowDirection TopDown" & ControlChars.Lf & _
            "False: FlowDirection LeftToRight")> _
         Public Property TopDown() As Boolean
            Get
               Return _TopDown
            End Get
            Set(ByVal NewValue As Boolean)
               _TopDown = NewValue
            End Set
         End Property

         Private _WrapContents As Boolean = Nothing
         <Description("True: Align controls in several lines." & ControlChars.Lf & _
            "False: enlarge heights to fill out the containing Sizerpanel")> _
         Public Property WrapContents() As Boolean
            Get
               Return _WrapContents
            End Get
            Set(ByVal NewValue As Boolean)
               _WrapContents = NewValue
            End Set
         End Property
         Private _AutoScroll As Boolean = Nothing
         <Description("If False: AutoSize-Behavior")> _
         Public Property AutoScroll() As Boolean
            Get
               Return _AutoScroll
            End Get
            Set(ByVal NewValue As Boolean)
               _AutoScroll = NewValue
            End Set
         End Property
         Private _Dock As DockStyle = Nothing
         Public Property Dock() As DockStyle
            Get
               Return _Dock
            End Get
            Set(ByVal NewValue As DockStyle)
               _Dock = NewValue
            End Set
         End Property
      End Class 'LayoutOptions

      Private Class Transfer
         Public OptionProp As PropertyDescriptor
         Public ControlProp As PropertyDescriptor
         Public OldValue As Object
      End Class

      Private Shared _Transfers As New List(Of Transfer)
      Shared Sub New()
         Dim tpControl = GetType(SizerPanel)
         Dim ControlProps = TypeDescriptor.GetProperties(tpControl)
         Dim tpOptions = GetType(LayoutOptions)
         For Each optProp As PropertyDescriptor In TypeDescriptor.GetProperties(tpOptions)
            Dim cProp = ControlProps(optProp.Name)
            If cProp.Null Then
               Throw New KeyNotFoundException(String.Concat( _
                  "Could not find the property """, tpControl.Name, ".", optProp.Name, _
                  """ corresponding to """, tpOptions.Name, ".", optProp.Name, """"))
            End If
            _Transfers.Add(New Transfer() With {.OptionProp = optProp, .ControlProp = cProp})
         Next
      End Sub

      Public Overrides Sub Initialize(ByVal component As IComponent)
         MyBase.Initialize(component)
         If Not GetType(SizerPanel).IsAssignableFrom(Control.GetType) Then
            Throw New ArgumentException(String.Concat( _
               "don't apply ", Me.GetType.Name, " to ", Control.GetType.Name))
         End If
         Verbs.Add(New DesignerVerb("Main Options", AddressOf MainOptions_Clicked))
      End Sub

      Public Sub MainOptions_Clicked(ByVal sender As Object, ByVal e As EventArgs)
         Dim options As New LayoutOptions
         For Each transfer In _Transfers
            With transfer
               'read from Control into options and additionally store value as "OldValue"
               .OldValue = .ControlProp.GetValue(Control)
               .OptionProp.SetValue(options, .OldValue)
            End With
         Next
         If OptionEditor.ShowDialog(Control.Name, options) <> DialogResult.OK Then Return
         For Each transfer In _Transfers
            With transfer
               ' read from options, check if changed, write to Control
               Dim newValue = .OptionProp.GetValue(options)
               If newValue.Equals(.OldValue) Then Continue For
               RaiseComponentChanging(.ControlProp)
               .ControlProp.SetValue(Control, newValue)
               RaiseComponentChanged(.ControlProp, .OldValue, newValue)
            End With
         Next
      End Sub

   End Class 'SizerPanelDesigner
End Namespace
